# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DCMINFO_METADATA = Metadata(
    id="f02767c67a910f0ef82176cc014707d1d17070a2.boutiques",
    name="dcminfo",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DcminfoTagParamsDictNoTag = typing.TypedDict('_DcminfoTagParamsDictNoTag', {
    "group": str,
    "element": str,
})
DcminfoTagParamsDictTagged = typing.TypedDict('DcminfoTagParamsDictTagged', {
    "@type": typing.Literal["tag"],
    "group": str,
    "element": str,
})
DcminfoTagParamsDict = _DcminfoTagParamsDictNoTag | DcminfoTagParamsDictTagged


_DcminfoConfigParamsDictNoTag = typing.TypedDict('_DcminfoConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DcminfoConfigParamsDictTagged = typing.TypedDict('DcminfoConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DcminfoConfigParamsDict = _DcminfoConfigParamsDictNoTag | DcminfoConfigParamsDictTagged


_DcminfoParamsDictNoTag = typing.TypedDict('_DcminfoParamsDictNoTag', {
    "all": bool,
    "csa": bool,
    "phoenix": bool,
    "tag": typing.NotRequired[list[DcminfoTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcminfoConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcminfoParamsDictTagged = typing.TypedDict('DcminfoParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dcminfo"],
    "all": bool,
    "csa": bool,
    "phoenix": bool,
    "tag": typing.NotRequired[list[DcminfoTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcminfoConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcminfoParamsDict = _DcminfoParamsDictNoTag | DcminfoParamsDictTagged


def dcminfo_tag(
    group: str,
    element: str,
) -> DcminfoTagParamsDictTagged:
    """
    Build parameters.
    
    Args:
        group: print field specified by the group & element tags supplied. Tags\
            should be supplied as Hexadecimal (i.e. as they appear in the -all\
            listing).
        element: print field specified by the group & element tags supplied.\
            Tags should be supplied as Hexadecimal (i.e. as they appear in the -all\
            listing).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "tag",
        "group": group,
        "element": element,
    }
    return params


def dcminfo_tag_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcminfoTagParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("group", None) is None:
        raise StyxValidationError("`group` must not be None")
    if not isinstance(params["group"], str):
        raise StyxValidationError(f'`group` has the wrong type: Received `{type(params.get("group", None))}` expected `str`')
    if params.get("element", None) is None:
        raise StyxValidationError("`element` must not be None")
    if not isinstance(params["element"], str):
        raise StyxValidationError(f'`element` has the wrong type: Received `{type(params.get("element", None))}` expected `str`')


def dcminfo_tag_cargs(
    params: DcminfoTagParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-tag")
    cargs.append(params.get("group", None))
    cargs.append(params.get("element", None))
    return cargs


def dcminfo_config(
    key: str,
    value: str,
) -> DcminfoConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dcminfo_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcminfoConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dcminfo_config_cargs(
    params: DcminfoConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DcminfoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DcminfoParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dcminfo_params(
    file: InputPathType,
    all_: bool = False,
    csa: bool = False,
    phoenix: bool = False,
    tag: list[DcminfoTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcminfoConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DcminfoParamsDictTagged:
    """
    Build parameters.
    
    Args:
        file: the DICOM file to be scanned.
        all_: print all DICOM fields.
        csa: print all Siemens CSA fields (excluding Phoenix unless requested).
        phoenix: print Siemens Phoenix protocol information.
        tag: print field specified by the group & element tags supplied. Tags\
            should be supplied as Hexadecimal (i.e. as they appear in the -all\
            listing).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dcminfo",
        "all": all_,
        "csa": csa,
        "phoenix": phoenix,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "file": file,
    }
    if tag is not None:
        params["tag"] = tag
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dcminfo_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcminfoParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("all", False) is None:
        raise StyxValidationError("`all` must not be None")
    if not isinstance(params["all"], bool):
        raise StyxValidationError(f'`all` has the wrong type: Received `{type(params.get("all", False))}` expected `bool`')
    if params.get("csa", False) is None:
        raise StyxValidationError("`csa` must not be None")
    if not isinstance(params["csa"], bool):
        raise StyxValidationError(f'`csa` has the wrong type: Received `{type(params.get("csa", False))}` expected `bool`')
    if params.get("phoenix", False) is None:
        raise StyxValidationError("`phoenix` must not be None")
    if not isinstance(params["phoenix"], bool):
        raise StyxValidationError(f'`phoenix` has the wrong type: Received `{type(params.get("phoenix", False))}` expected `bool`')
    if params.get("tag", None) is not None:
        if not isinstance(params["tag"], list):
            raise StyxValidationError(f'`tag` has the wrong type: Received `{type(params.get("tag", None))}` expected `list[DcminfoTagParamsDict] | None`')
        for e in params["tag"]:
            dcminfo_tag_validate(e)
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DcminfoConfigParamsDict] | None`')
        for e in params["config"]:
            dcminfo_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("file", None) is None:
        raise StyxValidationError("`file` must not be None")
    if not isinstance(params["file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `InputPathType`')


def dcminfo_cargs(
    params: DcminfoParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dcminfo")
    if params.get("all", False):
        cargs.append("-all")
    if params.get("csa", False):
        cargs.append("-csa")
    if params.get("phoenix", False):
        cargs.append("-phoenix")
    if params.get("tag", None) is not None:
        cargs.extend([a for c in [dcminfo_tag_cargs(s, execution) for s in params.get("tag", None)] for a in c])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dcminfo_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("file", None)))
    return cargs


def dcminfo_outputs(
    params: DcminfoParamsDict,
    execution: Execution,
) -> DcminfoOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DcminfoOutputs(
        root=execution.output_file("."),
    )
    return ret


def dcminfo_execute(
    params: DcminfoParamsDict,
    runner: Runner | None = None,
) -> DcminfoOutputs:
    """
    dcminfo
    
    Output DICOM fields in human-readable format.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcminfoOutputs`).
    """
    dcminfo_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DCMINFO_METADATA)
    params = execution.params(params)
    cargs = dcminfo_cargs(params, execution)
    ret = dcminfo_outputs(params, execution)
    execution.run(cargs)
    return ret


def dcminfo(
    file: InputPathType,
    all_: bool = False,
    csa: bool = False,
    phoenix: bool = False,
    tag: list[DcminfoTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcminfoConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DcminfoOutputs:
    """
    dcminfo
    
    Output DICOM fields in human-readable format.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        file: the DICOM file to be scanned.
        all_: print all DICOM fields.
        csa: print all Siemens CSA fields (excluding Phoenix unless requested).
        phoenix: print Siemens Phoenix protocol information.
        tag: print field specified by the group & element tags supplied. Tags\
            should be supplied as Hexadecimal (i.e. as they appear in the -all\
            listing).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcminfoOutputs`).
    """
    params = dcminfo_params(
        all_=all_,
        csa=csa,
        phoenix=phoenix,
        tag=tag,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        file=file,
    )
    return dcminfo_execute(params, runner)


__all__ = [
    "DCMINFO_METADATA",
    "DcminfoConfigParamsDict",
    "DcminfoConfigParamsDictTagged",
    "DcminfoOutputs",
    "DcminfoParamsDict",
    "DcminfoParamsDictTagged",
    "DcminfoTagParamsDict",
    "DcminfoTagParamsDictTagged",
    "dcminfo",
    "dcminfo_config",
    "dcminfo_execute",
    "dcminfo_params",
    "dcminfo_tag",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRCHECKERBOARDMASK_METADATA = Metadata(
    id="f1aa53a7b6030f421a6e8c4455b70ae4ec643303.boutiques",
    name="mrcheckerboardmask",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrcheckerboardmaskConfigParamsDictNoTag = typing.TypedDict('_MrcheckerboardmaskConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrcheckerboardmaskConfigParamsDictTagged = typing.TypedDict('MrcheckerboardmaskConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrcheckerboardmaskConfigParamsDict = _MrcheckerboardmaskConfigParamsDictNoTag | MrcheckerboardmaskConfigParamsDictTagged


_MrcheckerboardmaskParamsDictNoTag = typing.TypedDict('_MrcheckerboardmaskParamsDictNoTag', {
    "tiles": typing.NotRequired[int | None],
    "invert": bool,
    "nan": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcheckerboardmaskConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
MrcheckerboardmaskParamsDictTagged = typing.TypedDict('MrcheckerboardmaskParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrcheckerboardmask"],
    "tiles": typing.NotRequired[int | None],
    "invert": bool,
    "nan": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcheckerboardmaskConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
MrcheckerboardmaskParamsDict = _MrcheckerboardmaskParamsDictNoTag | MrcheckerboardmaskParamsDictTagged


def mrcheckerboardmask_config(
    key: str,
    value: str,
) -> MrcheckerboardmaskConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrcheckerboardmask_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcheckerboardmaskConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrcheckerboardmask_config_cargs(
    params: MrcheckerboardmaskConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrcheckerboardmaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrcheckerboardmaskParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output binary image mask."""


def mrcheckerboardmask_params(
    input_: InputPathType,
    output: str,
    tiles: int | None = None,
    invert: bool = False,
    nan: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcheckerboardmaskConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrcheckerboardmaskParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image to be used as a template.
        output: the output binary image mask.
        tiles: specify the number of tiles in any direction.
        invert: invert output binary mask.
        nan: use NaN as the output zero value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrcheckerboardmask",
        "invert": invert,
        "nan": nan,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if tiles is not None:
        params["tiles"] = tiles
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrcheckerboardmask_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcheckerboardmaskParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("tiles", None) is not None:
        if not isinstance(params["tiles"], int):
            raise StyxValidationError(f'`tiles` has the wrong type: Received `{type(params.get("tiles", None))}` expected `int | None`')
    if params.get("invert", False) is None:
        raise StyxValidationError("`invert` must not be None")
    if not isinstance(params["invert"], bool):
        raise StyxValidationError(f'`invert` has the wrong type: Received `{type(params.get("invert", False))}` expected `bool`')
    if params.get("nan", False) is None:
        raise StyxValidationError("`nan` must not be None")
    if not isinstance(params["nan"], bool):
        raise StyxValidationError(f'`nan` has the wrong type: Received `{type(params.get("nan", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrcheckerboardmaskConfigParamsDict] | None`')
        for e in params["config"]:
            mrcheckerboardmask_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrcheckerboardmask_cargs(
    params: MrcheckerboardmaskParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrcheckerboardmask")
    if params.get("tiles", None) is not None:
        cargs.extend([
            "-tiles",
            str(params.get("tiles", None))
        ])
    if params.get("invert", False):
        cargs.append("-invert")
    if params.get("nan", False):
        cargs.append("-nan")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrcheckerboardmask_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def mrcheckerboardmask_outputs(
    params: MrcheckerboardmaskParamsDict,
    execution: Execution,
) -> MrcheckerboardmaskOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrcheckerboardmaskOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrcheckerboardmask_execute(
    params: MrcheckerboardmaskParamsDict,
    runner: Runner | None = None,
) -> MrcheckerboardmaskOutputs:
    """
    mrcheckerboardmask
    
    Create bitwise checkerboard image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcheckerboardmaskOutputs`).
    """
    mrcheckerboardmask_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCHECKERBOARDMASK_METADATA)
    params = execution.params(params)
    cargs = mrcheckerboardmask_cargs(params, execution)
    ret = mrcheckerboardmask_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrcheckerboardmask(
    input_: InputPathType,
    output: str,
    tiles: int | None = None,
    invert: bool = False,
    nan: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcheckerboardmaskConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrcheckerboardmaskOutputs:
    """
    mrcheckerboardmask
    
    Create bitwise checkerboard image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image to be used as a template.
        output: the output binary image mask.
        tiles: specify the number of tiles in any direction.
        invert: invert output binary mask.
        nan: use NaN as the output zero value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcheckerboardmaskOutputs`).
    """
    params = mrcheckerboardmask_params(
        tiles=tiles,
        invert=invert,
        nan=nan,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return mrcheckerboardmask_execute(params, runner)


__all__ = [
    "MRCHECKERBOARDMASK_METADATA",
    "MrcheckerboardmaskConfigParamsDict",
    "MrcheckerboardmaskConfigParamsDictTagged",
    "MrcheckerboardmaskOutputs",
    "MrcheckerboardmaskParamsDict",
    "MrcheckerboardmaskParamsDictTagged",
    "mrcheckerboardmask",
    "mrcheckerboardmask_config",
    "mrcheckerboardmask_execute",
    "mrcheckerboardmask_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXELCORRESPONDENCE_METADATA = Metadata(
    id="0cdb6d5a0f6fc504a73335d26a91aed6bead44ba.boutiques",
    name="fixelcorrespondence",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_FixelcorrespondenceConfigParamsDictNoTag = typing.TypedDict('_FixelcorrespondenceConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
FixelcorrespondenceConfigParamsDictTagged = typing.TypedDict('FixelcorrespondenceConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
FixelcorrespondenceConfigParamsDict = _FixelcorrespondenceConfigParamsDictNoTag | FixelcorrespondenceConfigParamsDictTagged


_FixelcorrespondenceParamsDictNoTag = typing.TypedDict('_FixelcorrespondenceParamsDictNoTag', {
    "angle": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelcorrespondenceConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "subject_data": InputPathType,
    "template_directory": InputPathType,
    "output_directory": str,
    "output_data": str,
})
FixelcorrespondenceParamsDictTagged = typing.TypedDict('FixelcorrespondenceParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixelcorrespondence"],
    "angle": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelcorrespondenceConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "subject_data": InputPathType,
    "template_directory": InputPathType,
    "output_directory": str,
    "output_data": str,
})
FixelcorrespondenceParamsDict = _FixelcorrespondenceParamsDictNoTag | FixelcorrespondenceParamsDictTagged


def fixelcorrespondence_config(
    key: str,
    value: str,
) -> FixelcorrespondenceConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixelcorrespondence_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcorrespondenceConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixelcorrespondence_config_cargs(
    params: FixelcorrespondenceConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class FixelcorrespondenceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FixelcorrespondenceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fixelcorrespondence_params(
    subject_data: InputPathType,
    template_directory: InputPathType,
    output_directory: str,
    output_data: str,
    angle: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelcorrespondenceConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> FixelcorrespondenceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        subject_data: the input subject fixel data file. This should be a file\
            inside the fixel directory.
        template_directory: the input template fixel directory.
        output_directory: the fixel directory where the output file will be\
            written.
        output_data: the name of the output fixel data file. This will be\
            placed in the output fixel directory.
        angle: the max angle threshold for computing inter-subject fixel\
            correspondence (Default: 45 degrees).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixelcorrespondence",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "subject_data": subject_data,
        "template_directory": template_directory,
        "output_directory": output_directory,
        "output_data": output_data,
    }
    if angle is not None:
        params["angle"] = angle
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixelcorrespondence_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcorrespondenceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("angle", None) is not None:
        if not isinstance(params["angle"], (float, int)):
            raise StyxValidationError(f'`angle` has the wrong type: Received `{type(params.get("angle", None))}` expected `float | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[FixelcorrespondenceConfigParamsDict] | None`')
        for e in params["config"]:
            fixelcorrespondence_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("subject_data", None) is None:
        raise StyxValidationError("`subject_data` must not be None")
    if not isinstance(params["subject_data"], (pathlib.Path, str)):
        raise StyxValidationError(f'`subject_data` has the wrong type: Received `{type(params.get("subject_data", None))}` expected `InputPathType`')
    if params.get("template_directory", None) is None:
        raise StyxValidationError("`template_directory` must not be None")
    if not isinstance(params["template_directory"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template_directory` has the wrong type: Received `{type(params.get("template_directory", None))}` expected `InputPathType`')
    if params.get("output_directory", None) is None:
        raise StyxValidationError("`output_directory` must not be None")
    if not isinstance(params["output_directory"], str):
        raise StyxValidationError(f'`output_directory` has the wrong type: Received `{type(params.get("output_directory", None))}` expected `str`')
    if params.get("output_data", None) is None:
        raise StyxValidationError("`output_data` must not be None")
    if not isinstance(params["output_data"], str):
        raise StyxValidationError(f'`output_data` has the wrong type: Received `{type(params.get("output_data", None))}` expected `str`')


def fixelcorrespondence_cargs(
    params: FixelcorrespondenceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixelcorrespondence")
    if params.get("angle", None) is not None:
        cargs.extend([
            "-angle",
            str(params.get("angle", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixelcorrespondence_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("subject_data", None)))
    cargs.append(execution.input_file(params.get("template_directory", None)))
    cargs.append(params.get("output_directory", None))
    cargs.append(params.get("output_data", None))
    return cargs


def fixelcorrespondence_outputs(
    params: FixelcorrespondenceParamsDict,
    execution: Execution,
) -> FixelcorrespondenceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FixelcorrespondenceOutputs(
        root=execution.output_file("."),
    )
    return ret


def fixelcorrespondence_execute(
    params: FixelcorrespondenceParamsDict,
    runner: Runner | None = None,
) -> FixelcorrespondenceOutputs:
    """
    fixelcorrespondence
    
    Obtain fixel-fixel correpondence between a subject fixel image and a
    template fixel mask.
    
    It is assumed that the subject image has already been spatially normalised
    and is aligned with the template. The output fixel image will have the same
    fixels (and directions) of the template.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelcorrespondenceOutputs`).
    """
    fixelcorrespondence_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXELCORRESPONDENCE_METADATA)
    params = execution.params(params)
    cargs = fixelcorrespondence_cargs(params, execution)
    ret = fixelcorrespondence_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixelcorrespondence(
    subject_data: InputPathType,
    template_directory: InputPathType,
    output_directory: str,
    output_data: str,
    angle: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelcorrespondenceConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> FixelcorrespondenceOutputs:
    """
    fixelcorrespondence
    
    Obtain fixel-fixel correpondence between a subject fixel image and a
    template fixel mask.
    
    It is assumed that the subject image has already been spatially normalised
    and is aligned with the template. The output fixel image will have the same
    fixels (and directions) of the template.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        subject_data: the input subject fixel data file. This should be a file\
            inside the fixel directory.
        template_directory: the input template fixel directory.
        output_directory: the fixel directory where the output file will be\
            written.
        output_data: the name of the output fixel data file. This will be\
            placed in the output fixel directory.
        angle: the max angle threshold for computing inter-subject fixel\
            correspondence (Default: 45 degrees).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelcorrespondenceOutputs`).
    """
    params = fixelcorrespondence_params(
        angle=angle,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        subject_data=subject_data,
        template_directory=template_directory,
        output_directory=output_directory,
        output_data=output_data,
    )
    return fixelcorrespondence_execute(params, runner)


__all__ = [
    "FIXELCORRESPONDENCE_METADATA",
    "FixelcorrespondenceConfigParamsDict",
    "FixelcorrespondenceConfigParamsDictTagged",
    "FixelcorrespondenceOutputs",
    "FixelcorrespondenceParamsDict",
    "FixelcorrespondenceParamsDictTagged",
    "fixelcorrespondence",
    "fixelcorrespondence_config",
    "fixelcorrespondence_execute",
    "fixelcorrespondence_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKMAP_METADATA = Metadata(
    id="6c3a1a0072e37f35d493f0f168ff70a6ce77b20d.boutiques",
    name="tckmap",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TckmapVariousStringParamsDictNoTag = typing.TypedDict('_TckmapVariousStringParamsDictNoTag', {
    "obj": str,
})
TckmapVariousStringParamsDictTagged = typing.TypedDict('TckmapVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
TckmapVariousStringParamsDict = _TckmapVariousStringParamsDictNoTag | TckmapVariousStringParamsDictTagged


_TckmapVariousFileParamsDictNoTag = typing.TypedDict('_TckmapVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
TckmapVariousFileParamsDictTagged = typing.TypedDict('TckmapVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
TckmapVariousFileParamsDict = _TckmapVariousFileParamsDictNoTag | TckmapVariousFileParamsDictTagged


_TckmapConfigParamsDictNoTag = typing.TypedDict('_TckmapConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TckmapConfigParamsDictTagged = typing.TypedDict('TckmapConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TckmapConfigParamsDict = _TckmapConfigParamsDictNoTag | TckmapConfigParamsDictTagged


_TckmapParamsDictNoTag = typing.TypedDict('_TckmapParamsDictNoTag', {
    "template": typing.NotRequired[InputPathType | None],
    "vox": typing.NotRequired[list[float] | None],
    "datatype": typing.NotRequired[str | None],
    "dec": bool,
    "dixel": typing.NotRequired[typing.Union[TckmapVariousStringParamsDictTagged, TckmapVariousFileParamsDictTagged] | None],
    "tod": typing.NotRequired[int | None],
    "contrast": typing.NotRequired[str | None],
    "image": typing.NotRequired[InputPathType | None],
    "vector_file": typing.NotRequired[InputPathType | None],
    "stat_vox": typing.NotRequired[str | None],
    "stat_tck": typing.NotRequired[str | None],
    "fwhm_tck": typing.NotRequired[float | None],
    "map_zero": bool,
    "backtrack": bool,
    "upsample": typing.NotRequired[int | None],
    "precise": bool,
    "ends_only": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckmapConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "output": str,
})
TckmapParamsDictTagged = typing.TypedDict('TckmapParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tckmap"],
    "template": typing.NotRequired[InputPathType | None],
    "vox": typing.NotRequired[list[float] | None],
    "datatype": typing.NotRequired[str | None],
    "dec": bool,
    "dixel": typing.NotRequired[typing.Union[TckmapVariousStringParamsDictTagged, TckmapVariousFileParamsDictTagged] | None],
    "tod": typing.NotRequired[int | None],
    "contrast": typing.NotRequired[str | None],
    "image": typing.NotRequired[InputPathType | None],
    "vector_file": typing.NotRequired[InputPathType | None],
    "stat_vox": typing.NotRequired[str | None],
    "stat_tck": typing.NotRequired[str | None],
    "fwhm_tck": typing.NotRequired[float | None],
    "map_zero": bool,
    "backtrack": bool,
    "upsample": typing.NotRequired[int | None],
    "precise": bool,
    "ends_only": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckmapConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "output": str,
})
TckmapParamsDict = _TckmapParamsDictNoTag | TckmapParamsDictTagged


def tckmap_dixel_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": tckmap_various_string_cargs,
        "VariousFile": tckmap_various_file_cargs,
    }.get(t)


def tckmap_dixel_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": tckmap_various_string_validate,
        "VariousFile": tckmap_various_file_validate,
    }.get(t)


def tckmap_various_string(
    obj: str,
) -> TckmapVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def tckmap_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckmapVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def tckmap_various_string_cargs(
    params: TckmapVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def tckmap_various_file(
    obj: InputPathType,
) -> TckmapVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def tckmap_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckmapVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def tckmap_various_file_cargs(
    params: TckmapVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def tckmap_config(
    key: str,
    value: str,
) -> TckmapConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tckmap_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckmapConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tckmap_config_cargs(
    params: TckmapConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TckmapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TckmapParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output track-weighted image"""


def tckmap_params(
    tracks: InputPathType,
    output: str,
    template: InputPathType | None = None,
    vox: list[float] | None = None,
    datatype: str | None = None,
    dec: bool = False,
    dixel: typing.Union[TckmapVariousStringParamsDictTagged, TckmapVariousFileParamsDictTagged] | None = None,
    tod: int | None = None,
    contrast: str | None = None,
    image: InputPathType | None = None,
    vector_file: InputPathType | None = None,
    stat_vox: str | None = None,
    stat_tck: str | None = None,
    fwhm_tck: float | None = None,
    map_zero: bool = False,
    backtrack: bool = False,
    upsample: int | None = None,
    precise: bool = False,
    ends_only: bool = False,
    tck_weights_in: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckmapConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TckmapParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks: the input track file.
        output: the output track-weighted image.
        template: an image file to be used as a template for the output (the\
            output image will have the same transform and field of view).
        vox: provide either an isotropic voxel size (in mm), or comma-separated\
            list of 3 voxel dimensions.
        datatype: specify output image data type.
        dec: perform track mapping in directionally-encoded colour (DEC) space.
        dixel: map streamlines to dixels within each voxel; requires either a\
            number of dixels (references an internal direction set), or a path to a\
            text file containing a set of directions stored as azimuth/elevation\
            pairs.
        tod: generate a Track Orientation Distribution (TOD) in each voxel;\
            need to specify the maximum spherical harmonic degree lmax to use when\
            generating Apodised Point Spread Functions.
        contrast: define the desired form of contrast for the output image\
            Options are: tdi, length, invlength, scalar_map, scalar_map_count,\
            fod_amp, curvature, vector_file (default: tdi).
        image: provide the scalar image map for generating images with\
            'scalar_map' / 'scalar_map_count' contrast, or the spherical harmonics\
            image for 'fod_amp' contrast.
        vector_file: provide the vector data file for generating images with\
            'vector_file' contrast.
        stat_vox: define the statistic for choosing the final voxel intensities\
            for a given contrast type given the individual values from the tracks\
            passing through each voxel.\
            Options are: sum, min, mean, max (default: sum).
        stat_tck: define the statistic for choosing the contribution to be made\
            by each streamline as a function of the samples taken along their\
            lengths.\
            Only has an effect for 'scalar_map', 'fod_amp' and 'curvature'\
            contrast types.\
            Options are: sum, min, mean, max, median, mean_nonzero, gaussian,\
            ends_min, ends_mean, ends_max, ends_prod (default: mean).
        fwhm_tck: when using gaussian-smoothed per-track statistic, specify the\
            desired full-width half-maximum of the Gaussian smoothing kernel (in\
            mm).
        map_zero: if a streamline has zero contribution based on the contrast &\
            statistic, typically it is not mapped; use this option to still\
            contribute to the map even if this is the case (these non-contributing\
            voxels can then influence the mean value in each voxel of the map).
        backtrack: when using -stat_tck ends_*, if the streamline endpoint is\
            outside the FoV, backtrack along the streamline trajectory until an\
            appropriate point is found.
        upsample: upsample the tracks by some ratio using Hermite interpolation\
            before mappping\
            (If omitted, an appropriate ratio will be determined automatically).
        precise: use a more precise streamline mapping strategy, that\
            accurately quantifies the length through each voxel (these lengths are\
            then taken into account during TWI calculation).
        ends_only: only map the streamline endpoints to the image.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tckmap",
        "dec": dec,
        "map_zero": map_zero,
        "backtrack": backtrack,
        "precise": precise,
        "ends_only": ends_only,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks": tracks,
        "output": output,
    }
    if template is not None:
        params["template"] = template
    if vox is not None:
        params["vox"] = vox
    if datatype is not None:
        params["datatype"] = datatype
    if dixel is not None:
        params["dixel"] = dixel
    if tod is not None:
        params["tod"] = tod
    if contrast is not None:
        params["contrast"] = contrast
    if image is not None:
        params["image"] = image
    if vector_file is not None:
        params["vector_file"] = vector_file
    if stat_vox is not None:
        params["stat_vox"] = stat_vox
    if stat_tck is not None:
        params["stat_tck"] = stat_tck
    if fwhm_tck is not None:
        params["fwhm_tck"] = fwhm_tck
    if upsample is not None:
        params["upsample"] = upsample
    if tck_weights_in is not None:
        params["tck_weights_in"] = tck_weights_in
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tckmap_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckmapParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("vox", None) is not None:
        if not isinstance(params["vox"], list):
            raise StyxValidationError(f'`vox` has the wrong type: Received `{type(params.get("vox", None))}` expected `list[float] | None`')
        for e in params["vox"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`vox` has the wrong type: Received `{type(params.get("vox", None))}` expected `list[float] | None`')
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("dec", False) is None:
        raise StyxValidationError("`dec` must not be None")
    if not isinstance(params["dec"], bool):
        raise StyxValidationError(f'`dec` has the wrong type: Received `{type(params.get("dec", False))}` expected `bool`')
    if params.get("dixel", None) is not None:
        if not isinstance(params["dixel"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["dixel"])}\'')
        if "@type" not in params["dixel"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["dixel"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `dixel`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        tckmap_dixel_validate_dyn_fn(params["dixel"]["@type"])(params["dixel"])
    if params.get("tod", None) is not None:
        if not isinstance(params["tod"], int):
            raise StyxValidationError(f'`tod` has the wrong type: Received `{type(params.get("tod", None))}` expected `int | None`')
    if params.get("contrast", None) is not None:
        if not isinstance(params["contrast"], str):
            raise StyxValidationError(f'`contrast` has the wrong type: Received `{type(params.get("contrast", None))}` expected `str | None`')
    if params.get("image", None) is not None:
        if not isinstance(params["image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `InputPathType | None`')
    if params.get("vector_file", None) is not None:
        if not isinstance(params["vector_file"], (pathlib.Path, str)):
            raise StyxValidationError(f'`vector_file` has the wrong type: Received `{type(params.get("vector_file", None))}` expected `InputPathType | None`')
    if params.get("stat_vox", None) is not None:
        if not isinstance(params["stat_vox"], str):
            raise StyxValidationError(f'`stat_vox` has the wrong type: Received `{type(params.get("stat_vox", None))}` expected `str | None`')
    if params.get("stat_tck", None) is not None:
        if not isinstance(params["stat_tck"], str):
            raise StyxValidationError(f'`stat_tck` has the wrong type: Received `{type(params.get("stat_tck", None))}` expected `str | None`')
    if params.get("fwhm_tck", None) is not None:
        if not isinstance(params["fwhm_tck"], (float, int)):
            raise StyxValidationError(f'`fwhm_tck` has the wrong type: Received `{type(params.get("fwhm_tck", None))}` expected `float | None`')
    if params.get("map_zero", False) is None:
        raise StyxValidationError("`map_zero` must not be None")
    if not isinstance(params["map_zero"], bool):
        raise StyxValidationError(f'`map_zero` has the wrong type: Received `{type(params.get("map_zero", False))}` expected `bool`')
    if params.get("backtrack", False) is None:
        raise StyxValidationError("`backtrack` must not be None")
    if not isinstance(params["backtrack"], bool):
        raise StyxValidationError(f'`backtrack` has the wrong type: Received `{type(params.get("backtrack", False))}` expected `bool`')
    if params.get("upsample", None) is not None:
        if not isinstance(params["upsample"], int):
            raise StyxValidationError(f'`upsample` has the wrong type: Received `{type(params.get("upsample", None))}` expected `int | None`')
    if params.get("precise", False) is None:
        raise StyxValidationError("`precise` must not be None")
    if not isinstance(params["precise"], bool):
        raise StyxValidationError(f'`precise` has the wrong type: Received `{type(params.get("precise", False))}` expected `bool`')
    if params.get("ends_only", False) is None:
        raise StyxValidationError("`ends_only` must not be None")
    if not isinstance(params["ends_only"], bool):
        raise StyxValidationError(f'`ends_only` has the wrong type: Received `{type(params.get("ends_only", False))}` expected `bool`')
    if params.get("tck_weights_in", None) is not None:
        if not isinstance(params["tck_weights_in"], (pathlib.Path, str)):
            raise StyxValidationError(f'`tck_weights_in` has the wrong type: Received `{type(params.get("tck_weights_in", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TckmapConfigParamsDict] | None`')
        for e in params["config"]:
            tckmap_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def tckmap_cargs(
    params: TckmapParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tckmap")
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("vox", None) is not None:
        cargs.extend([
            "-vox",
            ",".join(map(str, params.get("vox", None)))
        ])
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("dec", False):
        cargs.append("-dec")
    if params.get("dixel", None) is not None:
        cargs.extend([
            "-dixel",
            *tckmap_dixel_cargs_dyn_fn(params.get("dixel", None)["@type"])(params.get("dixel", None), execution)
        ])
    if params.get("tod", None) is not None:
        cargs.extend([
            "-tod",
            str(params.get("tod", None))
        ])
    if params.get("contrast", None) is not None:
        cargs.extend([
            "-contrast",
            params.get("contrast", None)
        ])
    if params.get("image", None) is not None:
        cargs.extend([
            "-image",
            execution.input_file(params.get("image", None))
        ])
    if params.get("vector_file", None) is not None:
        cargs.extend([
            "-vector_file",
            execution.input_file(params.get("vector_file", None))
        ])
    if params.get("stat_vox", None) is not None:
        cargs.extend([
            "-stat_vox",
            params.get("stat_vox", None)
        ])
    if params.get("stat_tck", None) is not None:
        cargs.extend([
            "-stat_tck",
            params.get("stat_tck", None)
        ])
    if params.get("fwhm_tck", None) is not None:
        cargs.extend([
            "-fwhm_tck",
            str(params.get("fwhm_tck", None))
        ])
    if params.get("map_zero", False):
        cargs.append("-map_zero")
    if params.get("backtrack", False):
        cargs.append("-backtrack")
    if params.get("upsample", None) is not None:
        cargs.extend([
            "-upsample",
            str(params.get("upsample", None))
        ])
    if params.get("precise", False):
        cargs.append("-precise")
    if params.get("ends_only", False):
        cargs.append("-ends_only")
    if params.get("tck_weights_in", None) is not None:
        cargs.extend([
            "-tck_weights_in",
            execution.input_file(params.get("tck_weights_in", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tckmap_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks", None)))
    cargs.append(params.get("output", None))
    return cargs


def tckmap_outputs(
    params: TckmapParamsDict,
    execution: Execution,
) -> TckmapOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TckmapOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def tckmap_execute(
    params: TckmapParamsDict,
    runner: Runner | None = None,
) -> TckmapOutputs:
    """
    tckmap
    
    Use track data as a form of contrast for producing a high-resolution image.
    
    Note: if you run into limitations with RAM usage, make sure you output the
    results to a .mif file or .mih / .dat file pair - this will avoid the
    allocation of an additional buffer to store the output for write-out.
    
    References:
    
    * For TDI or DEC TDI:
    Calamante, F.; Tournier, J.-D.; Jackson, G. D. & Connelly, A. Track-density
    imaging (TDI): Super-resolution white matter imaging using whole-brain
    track-density mapping. NeuroImage, 2010, 53, 1233-1243
    
    * If using -contrast length and -stat_vox mean:
    Pannek, K.; Mathias, J. L.; Bigler, E. D.; Brown, G.; Taylor, J. D. & Rose,
    S. E. The average pathlength map: A diffusion MRI tractography-derived index
    for studying brain pathology. NeuroImage, 2011, 55, 133-141
    
    * If using -dixel option with TDI contrast only:
    Smith, R.E., Tournier, J-D., Calamante, F., Connelly, A. A novel paradigm
    for automated segmentation of very large whole-brain probabilistic
    tractography data sets. In proc. ISMRM, 2011, 19, 673
    
    * If using -dixel option with any other contrast:
    Pannek, K., Raffelt, D., Salvado, O., Rose, S. Incorporating directional
    information in diffusion tractography derived maps: angular track imaging
    (ATI). In Proc. ISMRM, 2012, 20, 1912
    
    * If using -tod option:
    Dhollander, T., Emsell, L., Van Hecke, W., Maes, F., Sunaert, S., Suetens,
    P. Track Orientation Density Imaging (TODI) and Track Orientation
    Distribution (TOD) based tractography. NeuroImage, 2014, 94, 312-336
    
    * If using other contrasts / statistics:
    Calamante, F.; Tournier, J.-D.; Smith, R. E. & Connelly, A. A generalised
    framework for super-resolution track-weighted imaging. NeuroImage, 2012, 59,
    2494-2503
    
    * If using -precise mapping option:
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312 (Appendix 3).
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckmapOutputs`).
    """
    tckmap_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKMAP_METADATA)
    params = execution.params(params)
    cargs = tckmap_cargs(params, execution)
    ret = tckmap_outputs(params, execution)
    execution.run(cargs)
    return ret


def tckmap(
    tracks: InputPathType,
    output: str,
    template: InputPathType | None = None,
    vox: list[float] | None = None,
    datatype: str | None = None,
    dec: bool = False,
    dixel: typing.Union[TckmapVariousStringParamsDictTagged, TckmapVariousFileParamsDictTagged] | None = None,
    tod: int | None = None,
    contrast: str | None = None,
    image: InputPathType | None = None,
    vector_file: InputPathType | None = None,
    stat_vox: str | None = None,
    stat_tck: str | None = None,
    fwhm_tck: float | None = None,
    map_zero: bool = False,
    backtrack: bool = False,
    upsample: int | None = None,
    precise: bool = False,
    ends_only: bool = False,
    tck_weights_in: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckmapConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TckmapOutputs:
    """
    tckmap
    
    Use track data as a form of contrast for producing a high-resolution image.
    
    Note: if you run into limitations with RAM usage, make sure you output the
    results to a .mif file or .mih / .dat file pair - this will avoid the
    allocation of an additional buffer to store the output for write-out.
    
    References:
    
    * For TDI or DEC TDI:
    Calamante, F.; Tournier, J.-D.; Jackson, G. D. & Connelly, A. Track-density
    imaging (TDI): Super-resolution white matter imaging using whole-brain
    track-density mapping. NeuroImage, 2010, 53, 1233-1243
    
    * If using -contrast length and -stat_vox mean:
    Pannek, K.; Mathias, J. L.; Bigler, E. D.; Brown, G.; Taylor, J. D. & Rose,
    S. E. The average pathlength map: A diffusion MRI tractography-derived index
    for studying brain pathology. NeuroImage, 2011, 55, 133-141
    
    * If using -dixel option with TDI contrast only:
    Smith, R.E., Tournier, J-D., Calamante, F., Connelly, A. A novel paradigm
    for automated segmentation of very large whole-brain probabilistic
    tractography data sets. In proc. ISMRM, 2011, 19, 673
    
    * If using -dixel option with any other contrast:
    Pannek, K., Raffelt, D., Salvado, O., Rose, S. Incorporating directional
    information in diffusion tractography derived maps: angular track imaging
    (ATI). In Proc. ISMRM, 2012, 20, 1912
    
    * If using -tod option:
    Dhollander, T., Emsell, L., Van Hecke, W., Maes, F., Sunaert, S., Suetens,
    P. Track Orientation Density Imaging (TODI) and Track Orientation
    Distribution (TOD) based tractography. NeuroImage, 2014, 94, 312-336
    
    * If using other contrasts / statistics:
    Calamante, F.; Tournier, J.-D.; Smith, R. E. & Connelly, A. A generalised
    framework for super-resolution track-weighted imaging. NeuroImage, 2012, 59,
    2494-2503
    
    * If using -precise mapping option:
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312 (Appendix 3).
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks: the input track file.
        output: the output track-weighted image.
        template: an image file to be used as a template for the output (the\
            output image will have the same transform and field of view).
        vox: provide either an isotropic voxel size (in mm), or comma-separated\
            list of 3 voxel dimensions.
        datatype: specify output image data type.
        dec: perform track mapping in directionally-encoded colour (DEC) space.
        dixel: map streamlines to dixels within each voxel; requires either a\
            number of dixels (references an internal direction set), or a path to a\
            text file containing a set of directions stored as azimuth/elevation\
            pairs.
        tod: generate a Track Orientation Distribution (TOD) in each voxel;\
            need to specify the maximum spherical harmonic degree lmax to use when\
            generating Apodised Point Spread Functions.
        contrast: define the desired form of contrast for the output image\
            Options are: tdi, length, invlength, scalar_map, scalar_map_count,\
            fod_amp, curvature, vector_file (default: tdi).
        image: provide the scalar image map for generating images with\
            'scalar_map' / 'scalar_map_count' contrast, or the spherical harmonics\
            image for 'fod_amp' contrast.
        vector_file: provide the vector data file for generating images with\
            'vector_file' contrast.
        stat_vox: define the statistic for choosing the final voxel intensities\
            for a given contrast type given the individual values from the tracks\
            passing through each voxel.\
            Options are: sum, min, mean, max (default: sum).
        stat_tck: define the statistic for choosing the contribution to be made\
            by each streamline as a function of the samples taken along their\
            lengths.\
            Only has an effect for 'scalar_map', 'fod_amp' and 'curvature'\
            contrast types.\
            Options are: sum, min, mean, max, median, mean_nonzero, gaussian,\
            ends_min, ends_mean, ends_max, ends_prod (default: mean).
        fwhm_tck: when using gaussian-smoothed per-track statistic, specify the\
            desired full-width half-maximum of the Gaussian smoothing kernel (in\
            mm).
        map_zero: if a streamline has zero contribution based on the contrast &\
            statistic, typically it is not mapped; use this option to still\
            contribute to the map even if this is the case (these non-contributing\
            voxels can then influence the mean value in each voxel of the map).
        backtrack: when using -stat_tck ends_*, if the streamline endpoint is\
            outside the FoV, backtrack along the streamline trajectory until an\
            appropriate point is found.
        upsample: upsample the tracks by some ratio using Hermite interpolation\
            before mappping\
            (If omitted, an appropriate ratio will be determined automatically).
        precise: use a more precise streamline mapping strategy, that\
            accurately quantifies the length through each voxel (these lengths are\
            then taken into account during TWI calculation).
        ends_only: only map the streamline endpoints to the image.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckmapOutputs`).
    """
    params = tckmap_params(
        template=template,
        vox=vox,
        datatype=datatype,
        dec=dec,
        dixel=dixel,
        tod=tod,
        contrast=contrast,
        image=image,
        vector_file=vector_file,
        stat_vox=stat_vox,
        stat_tck=stat_tck,
        fwhm_tck=fwhm_tck,
        map_zero=map_zero,
        backtrack=backtrack,
        upsample=upsample,
        precise=precise,
        ends_only=ends_only,
        tck_weights_in=tck_weights_in,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks=tracks,
        output=output,
    )
    return tckmap_execute(params, runner)


__all__ = [
    "TCKMAP_METADATA",
    "TckmapConfigParamsDict",
    "TckmapConfigParamsDictTagged",
    "TckmapOutputs",
    "TckmapParamsDict",
    "TckmapParamsDictTagged",
    "TckmapVariousFileParamsDict",
    "TckmapVariousFileParamsDictTagged",
    "TckmapVariousStringParamsDict",
    "TckmapVariousStringParamsDictTagged",
    "tckmap",
    "tckmap_config",
    "tckmap_execute",
    "tckmap_params",
    "tckmap_various_file",
    "tckmap_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARPINVERT_METADATA = Metadata(
    id="fe11b023d17d8a482f4f095709c5fdc089416cc2.boutiques",
    name="warpinvert",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_WarpinvertConfigParamsDictNoTag = typing.TypedDict('_WarpinvertConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
WarpinvertConfigParamsDictTagged = typing.TypedDict('WarpinvertConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
WarpinvertConfigParamsDict = _WarpinvertConfigParamsDictNoTag | WarpinvertConfigParamsDictTagged


_WarpinvertParamsDictNoTag = typing.TypedDict('_WarpinvertParamsDictNoTag', {
    "template": typing.NotRequired[InputPathType | None],
    "displacement": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpinvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "out": str,
})
WarpinvertParamsDictTagged = typing.TypedDict('WarpinvertParamsDictTagged', {
    "@type": typing.Literal["mrtrix/warpinvert"],
    "template": typing.NotRequired[InputPathType | None],
    "displacement": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpinvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "out": str,
})
WarpinvertParamsDict = _WarpinvertParamsDictNoTag | WarpinvertParamsDictTagged


def warpinvert_config(
    key: str,
    value: str,
) -> WarpinvertConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def warpinvert_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpinvertConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def warpinvert_config_cargs(
    params: WarpinvertConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class WarpinvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `WarpinvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output warp image."""


def warpinvert_params(
    in_: InputPathType,
    out: str,
    template: InputPathType | None = None,
    displacement: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpinvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> WarpinvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_: the input warp image.
        out: the output warp image.
        template: define a template image grid for the output warp.
        displacement: indicates that the input warp field is a displacement\
            field; the output will also be a displacement field.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/warpinvert",
        "displacement": displacement,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in": in_,
        "out": out,
    }
    if template is not None:
        params["template"] = template
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def warpinvert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpinvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("displacement", False) is None:
        raise StyxValidationError("`displacement` must not be None")
    if not isinstance(params["displacement"], bool):
        raise StyxValidationError(f'`displacement` has the wrong type: Received `{type(params.get("displacement", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[WarpinvertConfigParamsDict] | None`')
        for e in params["config"]:
            warpinvert_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in", None) is None:
        raise StyxValidationError("`in` must not be None")
    if not isinstance(params["in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in` has the wrong type: Received `{type(params.get("in", None))}` expected `InputPathType`')
    if params.get("out", None) is None:
        raise StyxValidationError("`out` must not be None")
    if not isinstance(params["out"], str):
        raise StyxValidationError(f'`out` has the wrong type: Received `{type(params.get("out", None))}` expected `str`')


def warpinvert_cargs(
    params: WarpinvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("warpinvert")
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("displacement", False):
        cargs.append("-displacement")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [warpinvert_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in", None)))
    cargs.append(params.get("out", None))
    return cargs


def warpinvert_outputs(
    params: WarpinvertParamsDict,
    execution: Execution,
) -> WarpinvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = WarpinvertOutputs(
        root=execution.output_file("."),
        out=execution.output_file(params.get("out", None)),
    )
    return ret


def warpinvert_execute(
    params: WarpinvertParamsDict,
    runner: Runner | None = None,
) -> WarpinvertOutputs:
    """
    warpinvert
    
    Invert a non-linear warp field.
    
    By default, this command assumes that the input warp field is a deformation
    field, i.e. each voxel stores the corresponding position in the other image
    (in scanner space), and the calculated output warp image will also be a
    deformation field. If the input warp field is instead a displacment field,
    i.e. where each voxel stores an offset from which to sample the other image
    (but still in scanner space), then the -displacement option should be used;
    the output warp field will additionally be calculated as a displacement
    field in this case.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpinvertOutputs`).
    """
    warpinvert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARPINVERT_METADATA)
    params = execution.params(params)
    cargs = warpinvert_cargs(params, execution)
    ret = warpinvert_outputs(params, execution)
    execution.run(cargs)
    return ret


def warpinvert(
    in_: InputPathType,
    out: str,
    template: InputPathType | None = None,
    displacement: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpinvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> WarpinvertOutputs:
    """
    warpinvert
    
    Invert a non-linear warp field.
    
    By default, this command assumes that the input warp field is a deformation
    field, i.e. each voxel stores the corresponding position in the other image
    (in scanner space), and the calculated output warp image will also be a
    deformation field. If the input warp field is instead a displacment field,
    i.e. where each voxel stores an offset from which to sample the other image
    (but still in scanner space), then the -displacement option should be used;
    the output warp field will additionally be calculated as a displacement
    field in this case.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_: the input warp image.
        out: the output warp image.
        template: define a template image grid for the output warp.
        displacement: indicates that the input warp field is a displacement\
            field; the output will also be a displacement field.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpinvertOutputs`).
    """
    params = warpinvert_params(
        template=template,
        displacement=displacement,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_=in_,
        out=out,
    )
    return warpinvert_execute(params, runner)


__all__ = [
    "WARPINVERT_METADATA",
    "WarpinvertConfigParamsDict",
    "WarpinvertConfigParamsDictTagged",
    "WarpinvertOutputs",
    "WarpinvertParamsDict",
    "WarpinvertParamsDictTagged",
    "warpinvert",
    "warpinvert_config",
    "warpinvert_execute",
    "warpinvert_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWIGRADCHECK_METADATA = Metadata(
    id="efa3ccbad97d0c5bc080c7e11292722025c2c985.boutiques",
    name="dwigradcheck",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DwigradcheckFslgradParamsDictNoTag = typing.TypedDict('_DwigradcheckFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwigradcheckFslgradParamsDictTagged = typing.TypedDict('DwigradcheckFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwigradcheckFslgradParamsDict = _DwigradcheckFslgradParamsDictNoTag | DwigradcheckFslgradParamsDictTagged


_DwigradcheckExportGradFslParamsDictNoTag = typing.TypedDict('_DwigradcheckExportGradFslParamsDictNoTag', {
    "bvecs_path": str,
    "bvals_path": str,
})
DwigradcheckExportGradFslParamsDictTagged = typing.TypedDict('DwigradcheckExportGradFslParamsDictTagged', {
    "@type": typing.Literal["export_grad_fsl"],
    "bvecs_path": str,
    "bvals_path": str,
})
DwigradcheckExportGradFslParamsDict = _DwigradcheckExportGradFslParamsDictNoTag | DwigradcheckExportGradFslParamsDictTagged


_DwigradcheckParamsDictNoTag = typing.TypedDict('_DwigradcheckParamsDictNoTag', {
    "input_image": InputPathType,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwigradcheckFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "number": typing.NotRequired[int | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[DwigradcheckExportGradFslParamsDict | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[str] | None],
    "help": bool,
    "version": bool,
})
DwigradcheckParamsDictTagged = typing.TypedDict('DwigradcheckParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwigradcheck"],
    "input_image": InputPathType,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwigradcheckFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "number": typing.NotRequired[int | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[DwigradcheckExportGradFslParamsDict | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[str] | None],
    "help": bool,
    "version": bool,
})
DwigradcheckParamsDict = _DwigradcheckParamsDictNoTag | DwigradcheckParamsDictTagged


def dwigradcheck_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> DwigradcheckFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwigradcheck_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwigradcheckFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwigradcheck_fslgrad_cargs(
    params: DwigradcheckFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


class DwigradcheckExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwigradcheckExportGradFslParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


def dwigradcheck_export_grad_fsl(
    bvecs_path: str,
    bvals_path: str,
) -> DwigradcheckExportGradFslParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
        bvals_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "export_grad_fsl",
        "bvecs_path": bvecs_path,
        "bvals_path": bvals_path,
    }
    return params


def dwigradcheck_export_grad_fsl_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwigradcheckExportGradFslParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs_path", None) is None:
        raise StyxValidationError("`bvecs_path` must not be None")
    if not isinstance(params["bvecs_path"], str):
        raise StyxValidationError(f'`bvecs_path` has the wrong type: Received `{type(params.get("bvecs_path", None))}` expected `str`')
    if params.get("bvals_path", None) is None:
        raise StyxValidationError("`bvals_path` must not be None")
    if not isinstance(params["bvals_path"], str):
        raise StyxValidationError(f'`bvals_path` has the wrong type: Received `{type(params.get("bvals_path", None))}` expected `str`')


def dwigradcheck_export_grad_fsl_cargs(
    params: DwigradcheckExportGradFslParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-export_grad_fsl")
    cargs.append(params.get("bvecs_path", None))
    cargs.append(params.get("bvals_path", None))
    return cargs


def dwigradcheck_export_grad_fsl_outputs(
    params: DwigradcheckExportGradFslParamsDict,
    execution: Execution,
) -> DwigradcheckExportGradFslOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwigradcheckExportGradFslOutputs(
        root=execution.output_file("."),
        bvecs_path=execution.output_file(params.get("bvecs_path", None)),
        bvals_path=execution.output_file(params.get("bvals_path", None)),
    )
    return ret


class DwigradcheckOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwigradcheckParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_grad_fsl: DwigradcheckExportGradFslOutputs | None
    """Outputs from `dwigradcheck_export_grad_fsl_outputs`."""


def dwigradcheck_params(
    input_image: InputPathType,
    grad: InputPathType | None = None,
    fslgrad: DwigradcheckFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    number: int | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: DwigradcheckExportGradFslParamsDict | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DwigradcheckParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_image: The input DWI series to be checked.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Provide a brain mask image.
        number: Set the number of tracks to generate for each test.
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwigradcheck",
        "input_image": input_image,
        "nocleanup": nocleanup,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if mask_image is not None:
        params["mask_image"] = mask_image
    if number is not None:
        params["number"] = number
    if export_grad_mrtrix is not None:
        params["export_grad_mrtrix"] = export_grad_mrtrix
    if export_grad_fsl is not None:
        params["export_grad_fsl"] = export_grad_fsl
    if scratch_dir is not None:
        params["scratch_dir"] = scratch_dir
    if continue_scratch_dir is not None:
        params["continue_scratch_dir"] = continue_scratch_dir
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dwigradcheck_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwigradcheckParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwigradcheck_fslgrad_validate(params["fslgrad"])
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("number", None) is not None:
        if not isinstance(params["number"], int):
            raise StyxValidationError(f'`number` has the wrong type: Received `{type(params.get("number", None))}` expected `int | None`')
    if params.get("export_grad_mrtrix", None) is not None:
        if not isinstance(params["export_grad_mrtrix"], str):
            raise StyxValidationError(f'`export_grad_mrtrix` has the wrong type: Received `{type(params.get("export_grad_mrtrix", None))}` expected `str | None`')
    if params.get("export_grad_fsl", None) is not None:
        dwigradcheck_export_grad_fsl_validate(params["export_grad_fsl"])
    if params.get("nocleanup", False) is None:
        raise StyxValidationError("`nocleanup` must not be None")
    if not isinstance(params["nocleanup"], bool):
        raise StyxValidationError(f'`nocleanup` has the wrong type: Received `{type(params.get("nocleanup", False))}` expected `bool`')
    if params.get("scratch_dir", None) is not None:
        if not isinstance(params["scratch_dir"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scratch_dir` has the wrong type: Received `{type(params.get("scratch_dir", None))}` expected `InputPathType | None`')
    if params.get("continue_scratch_dir", None) is not None:
        if not isinstance(params["continue_scratch_dir"], list):
            raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
        if len(params["continue_scratch_dir"]) != 2:
            raise StyxValidationError("Parameter `continue_scratch_dir` must contain exactly 2 elements")
        for e in params["continue_scratch_dir"]:
            if not isinstance(e, (pathlib.Path, str)):
                raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], (float, int)):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `float | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[str] | None`')
        for e in params["config"]:
            if not isinstance(e, str):
                raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[str] | None`')
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')


def dwigradcheck_cargs(
    params: DwigradcheckParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwigradcheck")
    cargs.append(execution.input_file(params.get("input_image", None)))
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwigradcheck_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("number", None) is not None:
        cargs.extend([
            "-number",
            str(params.get("number", None))
        ])
    if params.get("export_grad_mrtrix", None) is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            params.get("export_grad_mrtrix", None)
        ])
    if params.get("export_grad_fsl", None) is not None:
        cargs.extend(dwigradcheck_export_grad_fsl_cargs(params.get("export_grad_fsl", None), execution))
    if params.get("nocleanup", False):
        cargs.append("-nocleanup")
    if params.get("scratch_dir", None) is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(params.get("scratch_dir", None))
        ])
    if params.get("continue_scratch_dir", None) is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in params.get("continue_scratch_dir", None)]
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([
            "-config",
            *params.get("config", None)
        ])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    return cargs


def dwigradcheck_outputs(
    params: DwigradcheckParamsDict,
    execution: Execution,
) -> DwigradcheckOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwigradcheckOutputs(
        root=execution.output_file("."),
        export_grad_mrtrix=execution.output_file(params.get("export_grad_mrtrix", None)) if (params.get("export_grad_mrtrix") is not None) else None,
        export_grad_fsl=dwigradcheck_export_grad_fsl_outputs(params.get("export_grad_fsl"), execution) if params.get("export_grad_fsl") else None,
    )
    return ret


def dwigradcheck_execute(
    params: DwigradcheckParamsDict,
    runner: Runner | None = None,
) -> DwigradcheckOutputs:
    """
    dwigradcheck
    
    Check the orientation of the diffusion gradient table.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwigradcheckOutputs`).
    """
    dwigradcheck_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWIGRADCHECK_METADATA)
    params = execution.params(params)
    cargs = dwigradcheck_cargs(params, execution)
    ret = dwigradcheck_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwigradcheck(
    input_image: InputPathType,
    grad: InputPathType | None = None,
    fslgrad: DwigradcheckFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    number: int | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: DwigradcheckExportGradFslParamsDict | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DwigradcheckOutputs:
    """
    dwigradcheck
    
    Check the orientation of the diffusion gradient table.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_image: The input DWI series to be checked.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Provide a brain mask image.
        number: Set the number of tracks to generate for each test.
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwigradcheckOutputs`).
    """
    params = dwigradcheck_params(
        input_image=input_image,
        grad=grad,
        fslgrad=fslgrad,
        mask_image=mask_image,
        number=number,
        export_grad_mrtrix=export_grad_mrtrix,
        export_grad_fsl=export_grad_fsl,
        nocleanup=nocleanup,
        scratch_dir=scratch_dir,
        continue_scratch_dir=continue_scratch_dir,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
    )
    return dwigradcheck_execute(params, runner)


__all__ = [
    "DWIGRADCHECK_METADATA",
    "DwigradcheckExportGradFslOutputs",
    "DwigradcheckExportGradFslParamsDict",
    "DwigradcheckExportGradFslParamsDictTagged",
    "DwigradcheckFslgradParamsDict",
    "DwigradcheckFslgradParamsDictTagged",
    "DwigradcheckOutputs",
    "DwigradcheckParamsDict",
    "DwigradcheckParamsDictTagged",
    "dwigradcheck",
    "dwigradcheck_execute",
    "dwigradcheck_export_grad_fsl",
    "dwigradcheck_fslgrad",
    "dwigradcheck_params",
]
